@@ -22,14 +22,14 @@
 #include "grid_point_search.h"
 #include "cdo_options.h"
 #include "progress.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "matrix_view.h"
 
 template <typename T, typename CMP_FUNC>
 T
-fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T missval, MatrixView<T> &matrix1, CMP_FUNC is_EQ)
+fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T missval, MatrixView<T> &matrix1, CMP_FUNC is_NE)
 {
-  if (!is_EQ(matrix1[j][i], missval)) return matrix1[j][i];
+  if (is_NE(matrix1[j][i], missval)) return matrix1[j][i];
 
   T rval = missval;
   long ir, iu, il, io;
@@ -40,7 +40,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
   double xr = 0.0, xu = 0.0, xl = 0.0, xo = 0.0;
 
   for (ir = i + 1; ir < nx; ir++)
-    if (!is_EQ(matrix1[j][ir], missval))
+    if (is_NE(matrix1[j][ir], missval))
       {
         kr = ir - i;
         xr = matrix1[j][ir];
@@ -50,7 +50,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
   if (globgrid && ir == nx)
     {
       for (ir = 0; ir < i; ir++)
-        if (!is_EQ(matrix1[j][ir], missval))
+        if (is_NE(matrix1[j][ir], missval))
           {
             kr = nx + ir - i;
             xr = matrix1[j][ir];
@@ -59,7 +59,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
     }
 
   for (il = i - 1; il >= 0; il--)
-    if (!is_EQ(matrix1[j][il], missval))
+    if (is_NE(matrix1[j][il], missval))
       {
         kl = i - il;
         xl = matrix1[j][il];
@@ -69,7 +69,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
   if (globgrid && il == -1)
     {
       for (il = nx - 1; il > i; il--)
-        if (!is_EQ(matrix1[j][il], missval))
+        if (is_NE(matrix1[j][il], missval))
           {
             kl = nx + i - il;
             xl = matrix1[j][il];
@@ -78,7 +78,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
     }
 
   for (iu = j + 1; iu < ny; iu++)
-    if (!is_EQ(matrix1[iu][i], missval))
+    if (is_NE(matrix1[iu][i], missval))
       {
         ku = iu - j;
         xu = matrix1[iu][i];
@@ -86,7 +86,7 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
       }
 
   for (io = j - 1; io >= 0; io--)
-    if (!is_EQ(matrix1[io][i], missval))
+    if (is_NE(matrix1[io][i], missval))
       {
         ko = j - io;
         xo = matrix1[io][i];
@@ -123,10 +123,11 @@ fillmiss_kernel(int nfill, bool globgrid, long nx, long ny, long i, long j, T mi
   return rval;
 }
 
-template <typename T, typename CMP_FUNC>
+template <typename T1, typename T2, typename CMP_FUNC>
 void
-fillmiss_x(int gridID, Varray<T> &vIn, Varray<T> &vOut, T missval, int nfill, CMP_FUNC is_EQ)
+fillmiss_x(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, double mv, int nfill, CMP_FUNC is_NE)
 {
+  T1 missval = mv;
   long nx = gridInqXsize(gridID);
   long ny = gridInqYsize(gridID);
   auto globgrid = (bool) gridIsCircular(gridID);
@@ -134,42 +135,29 @@ fillmiss_x(int gridID, Varray<T> &vIn, Varray<T> &vOut, T missval, int nfill, CM
   auto gridtype = gridInqType(gridID);
   if (!(gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)) cdo_abort("Unsupported grid type: %s!", gridNamePtr(gridtype));
 
-  MatrixView<T> matrix1(vIn.data(), ny, nx);
-  MatrixView<T> matrix2(vOut.data(), ny, nx);
+  MatrixView<T1> matrix1(vIn.data(), ny, nx);
+  MatrixView<T2> matrix2(vOut.data(), ny, nx);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
 #endif
   for (long j = 0; j < ny; ++j)
-    for (long i = 0; i < nx; ++i) { matrix2[j][i] = fillmiss_kernel(nfill, globgrid, nx, ny, i, j, missval, matrix1, is_EQ); }
+    for (long i = 0; i < nx; ++i) { matrix2[j][i] = fillmiss_kernel(nfill, globgrid, nx, ny, i, j, missval, matrix1, is_NE); }
 }
 
 static void
 fillmiss(Field &field1, Field &field2, int nfill)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (std::isnan(field1.missval))
-    {
-      if (field1.memType == MemType::Float)
-        fillmiss_x(field1.grid, field1.vec_f, field2.vec_f, (float) field1.missval, nfill, dbl_is_equal);
-      else
-        fillmiss_x(field1.grid, field1.vec_d, field2.vec_d, field1.missval, nfill, dbl_is_equal);
-    }
-  else
-    {
-      if (field1.memType == MemType::Float)
-        fillmiss_x(field1.grid, field1.vec_f, field2.vec_f, (float) field1.missval, nfill, is_equal);
-      else
-        fillmiss_x(field1.grid, field1.vec_d, field2.vec_d, field1.missval, nfill, is_equal);
-    }
+  auto func = [&](auto &v1, auto &v2, auto grid, double mv, auto is_NE) { fillmiss_x(v1, v2, grid, mv, nfill, is_NE); };
+  std::isnan(field1.missval) ? field_operation2(func, field1, field2, field1.grid, field1.missval, fp_is_not_equal)
+                             : field_operation2(func, field1, field2, field1.grid, field1.missval, is_not_equal);
 }
 
 template <typename T, typename CMP_FUNC>
 T
-fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView<T> &matrix1, CMP_FUNC is_EQ)
+fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView<T> &matrix1, CMP_FUNC is_NE)
 {
-  if (!is_EQ(matrix1[j][i], missval)) return matrix1[j][i];
+  if (is_NE(matrix1[j][i], missval)) return matrix1[j][i];
 
   T rval = missval;
   long ir, iu, il, io;
@@ -180,7 +168,7 @@ fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView
   T xr = 0.0, xu = 0.0, xl = 0.0, xo = 0.0;
 
   for (ir = i + 1; ir < nx; ir++)
-    if (!is_EQ(matrix1[j][ir], missval))
+    if (is_NE(matrix1[j][ir], missval))
       {
         kr = ir - i;
         xr = matrix1[j][ir];
@@ -188,7 +176,7 @@ fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView
       }
 
   for (il = i - 1; il >= 0; il--)
-    if (!is_EQ(matrix1[j][il], missval))
+    if (is_NE(matrix1[j][il], missval))
       {
         kl = i - il;
         xl = matrix1[j][il];
@@ -196,7 +184,7 @@ fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView
       }
 
   for (iu = j + 1; iu < ny; iu++)
-    if (!is_EQ(matrix1[iu][i], missval))
+    if (is_NE(matrix1[iu][i], missval))
       {
         ku = iu - j;
         xu = matrix1[iu][i];
@@ -204,7 +192,7 @@ fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView
       }
 
   for (io = j - 1; io >= 0; io--)
-    if (!is_EQ(matrix1[io][i], missval))
+    if (is_NE(matrix1[io][i], missval))
       {
         ko = j - io;
         xo = matrix1[io][i];
@@ -234,15 +222,16 @@ fillmiss_one_step_kernel(long nx, long ny, long i, long j, T missval, MatrixView
   return rval;
 }
 
-template <typename T, typename CMP_FUNC>
+template <typename T1, typename T2, typename CMP_FUNC>
 void
-fillmiss_one_step_x(int gridID, Varray<T> &vIn, Varray<T> &vOut, T missval, int maxfill, CMP_FUNC is_EQ)
+fillmiss_one_step_x(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, double mv, int maxfill, CMP_FUNC is_EQ)
 {
+  T1 missval = mv;
   long nx = gridInqXsize(gridID);
   long ny = gridInqYsize(gridID);
 
-  MatrixView<T> matrix1(vIn.data(), ny, nx);
-  MatrixView<T> matrix2(vOut.data(), ny, nx);
+  MatrixView<T1> matrix1(vIn.data(), ny, nx);
+  MatrixView<T2> matrix2(vOut.data(), ny, nx);
 
   for (int fill_iterations = 0; fill_iterations < maxfill; fill_iterations++)
     {
@@ -261,32 +250,20 @@ fillmiss_one_step_x(int gridID, Varray<T> &vIn, Varray<T> &vOut, T missval, int
 static void
 fillmiss_one_step(Field &field1, Field &field2, int maxfill)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (std::isnan(field1.missval))
-    {
-      if (field1.memType == MemType::Float)
-        fillmiss_one_step_x(field1.grid, field1.vec_f, field2.vec_f, (float) field1.missval, maxfill, dbl_is_equal);
-      else
-        fillmiss_one_step_x(field1.grid, field1.vec_d, field2.vec_d, field1.missval, maxfill, dbl_is_equal);
-    }
-  else
-    {
-      if (field1.memType == MemType::Float)
-        fillmiss_one_step_x(field1.grid, field1.vec_f, field2.vec_f, (float) field1.missval, maxfill, is_equal);
-      else
-        fillmiss_one_step_x(field1.grid, field1.vec_d, field2.vec_d, field1.missval, maxfill, is_equal);
-    }
+  auto func = [&](auto &v1, auto &v2, auto grid, double mv, auto is_NE) { fillmiss_one_step_x(v1, v2, grid, mv, maxfill, is_NE); };
+  std::isnan(field1.missval) ? field_operation2(func, field1, field2, field1.grid, field1.missval, fp_is_not_equal)
+                             : field_operation2(func, field1, field2, field1.grid, field1.missval, is_not_equal);
 }
 
-template <typename T>
+template <typename T1, typename T2>
 void
-setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missval, int numNeighbors)
+setmisstodis(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, size_t numMissVals, double mv, int numNeighbors)
 {
+  T1 missval = mv;
   auto gridID0 = gridID;
 
   auto gridsize = gridInqSize(gridID);
-  auto nvals = gridsize - nmiss;
+  auto nvals = gridsize - numMissVals;
   gridID = generate_full_point_grid(gridID);
 
   if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
@@ -307,14 +284,14 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
   cdo_grid_to_radian(gridID, CDI_XAXIS, xvals, "grid center lon");
   cdo_grid_to_radian(gridID, CDI_YAXIS, yvals, "grid center lat");
 
-  std::vector<size_t> mindex(nmiss, 1), vindex(nvals, 1);
+  std::vector<size_t> mindex(numMissVals, 1), vindex(nvals, 1);
   Varray<double> lons(nvals), lats(nvals);
 
   size_t nv = 0, nm = 0;
   for (size_t i = 0; i < gridsize; ++i)
     {
       vOut[i] = vIn[i];
-      if (dbl_is_equal(vIn[i], missval))
+      if (fp_is_equal(vIn[i], missval))
         {
           mindex[nm] = i;
           nm++;
@@ -340,7 +317,7 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
 
   GridPointSearch gps;
 
-  if (nmiss)
+  if (numMissVals)
     {
       auto xIsCyclic = false;
       size_t dims[2] = { nvals, 0 };
@@ -350,7 +327,7 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
 
   if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", timer.elapsed());
 
-  progress::init();
+  cdo::Progress progress;
 
   timer.reset();
 
@@ -359,10 +336,10 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
 #endif
-  for (size_t i = 0; i < nmiss; ++i)
+  for (size_t i = 0; i < numMissVals; ++i)
     {
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double) atomicCount / nmiss);
+      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / numMissVals);
 
       auto ompthID = cdo_omp_get_thread_num();
 
@@ -373,13 +350,12 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
       if (nadds)
         {
           double result = 0.0;
-          for (size_t n = 0; n < nadds; ++n) result += vIn[vindex[knnWeights[ompthID].m_indices[n]]] * knnWeights[ompthID].m_dist[n];
+          for (size_t n = 0; n < nadds; ++n)
+            result += vIn[vindex[knnWeights[ompthID].m_indices[n]]] * knnWeights[ompthID].m_dist[n];
           vOut[mindex[i]] = result;
         }
     }
 
-  progress::update(0, 1, 1);
-
   if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds", timer.elapsed());
 
   grid_point_search_delete(gps);
@@ -390,16 +366,30 @@ setmisstodis(size_t nmiss, int gridID, Varray<T> &vIn, Varray<T> &vOut, T missva
 static void
 setmisstodis(Field &field1, Field &field2, int numNeighbors)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (field1.memType == MemType::Float)
-    setmisstodis(field1.nmiss, field1.grid, field1.vec_f, field2.vec_f, (float) field1.missval, numNeighbors);
-  else
-    setmisstodis(field1.nmiss, field1.grid, field1.vec_d, field2.vec_d, field1.missval, numNeighbors);
+  auto func = [&](auto &v1, auto &v2, auto grid, auto numMissVals, double mv) {
+    setmisstodis(v1, v2, grid, numMissVals, mv, numNeighbors);
+  };
+  field_operation2(func, field1, field2, field1.grid, field1.numMissVals, field1.missval);
 }
 
-class ModuleFillmiss
+class Fillmiss : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Fillmiss",
+    .operators = { { "fillmiss", 0, 0, "nfill" },
+                   { "fillmiss2", 0, 0, "nfill" },
+                   { "setmisstonn", 0, 0, "", SetmissHelp },
+                   { "setmisstodis", 0, 0, "numberofneighbors", SetmissHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Fillmiss> registration = RegisterEntry<Fillmiss>(module);
+
+  int FILLMISS, FILLMISS2, SETMISSTONN, SETMISSTODIS;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -413,26 +403,23 @@ class ModuleFillmiss
   Field field1, field2;
 
   void (*fill_method)(Field &, Field &, int) = &setmisstodis;
-  int FILLMISS, FILLMISS2, SETMISSTONN, SETMISSTODIS;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-    FILLMISS        = cdo_operator_add("fillmiss"   ,   0, 0, "nfill");
-    FILLMISS2 = cdo_operator_add("fillmiss2"  ,   0, 0, "nfill");
-    SETMISSTONN     = cdo_operator_add("setmisstonn" ,  0, 0, "");
-    SETMISSTODIS    = cdo_operator_add("setmisstodis" , 0, 0, "number of neighbors");
+    FILLMISS = module.get_id("fillmiss");
+    FILLMISS2 = module.get_id("fillmiss2");
+    SETMISSTONN = module.get_id("setmisstonn");
+    SETMISSTODIS = module.get_id("setmisstodis");
 
     operatorID = cdo_operator_id();
 
-    if      (operatorID == FILLMISS)        fill_method = &fillmiss;
-    else if (operatorID == FILLMISS2) fill_method = &fillmiss_one_step;
-    else if (operatorID == SETMISSTONN)     fill_method = &setmisstodis;
-    else if (operatorID == SETMISSTODIS)    fill_method = &setmisstodis;
+    // clang-format off
+    if      (operatorID == FILLMISS)      fill_method = &fillmiss;
+    else if (operatorID == FILLMISS2)     fill_method = &fillmiss_one_step;
+    else if (operatorID == SETMISSTONN)   fill_method = &setmisstodis;
+    else if (operatorID == SETMISSTODIS)  fill_method = &setmisstodis;
     // clang-format on
 
     nfill = (operatorID == SETMISSTODIS) ? 4 : 1;
@@ -459,11 +446,11 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -479,24 +466,24 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            field1.init(varList1[varID]);
+            const auto &var1 = varList1.vars[varID];
+            field1.init(var1);
             cdo_read_record(streamID1, field1);
 
             cdo_def_record(streamID2, varID, levelID);
 
-            if (field1.nmiss == 0 || field1.nmiss == varList1[varID].gridsize) { cdo_write_record(streamID2, field1); }
+            if (field1.numMissVals == 0 || field1.numMissVals == var1.gridsize) { cdo_write_record(streamID2, field1); }
             else
               {
-                auto gridtype = gridInqType(varList1[varID].gridID);
-                if ((operatorID == FILLMISS || operatorID == FILLMISS2)
-                    && (gridtype == GRID_GME || gridtype == GRID_UNSTRUCTURED))
+                auto gridtype = var1.gridType;
+                if ((operatorID == FILLMISS || operatorID == FILLMISS2) && (gridtype == GRID_GME || gridtype == GRID_UNSTRUCTURED))
                   cdo_abort("%s data unsupported!", gridNamePtr(gridtype));
 
-                field2.init(varList1[varID]);
+                field2.init(var1);
 
                 fill_method(field1, field2, nfill);
 
-                field2.nmiss = field_num_mv(field2);
+                field2.numMissVals = field_num_mv(field2);
 
                 cdo_write_record(streamID2, field2);
               }
@@ -507,22 +494,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Fillmiss(void *process)
-{
-  ModuleFillmiss fillmiss;
-  fillmiss.init(process);
-  fillmiss.run();
-  fillmiss.close();
-
-  return nullptr;
-}
