@@ -161,14 +161,14 @@ public:
             numMissVals = 0;
             for (size_t i = 0; i < gridsize; ++i)
               {
-                if (!DBL_IS_EQUAL(array[i], missval))
+                if (!fp_is_equal(array[i], missval))
                   {
                     vartcount[varID][i] += 1;
                     int index = 0;
                     while (index < nbins)
                       {
                         auto offset = gridsize * index;
-                        if (!DBL_IS_EQUAL(vardata[varID][offset + i], missval) && array[i] >= fltarr[index]
+                        if (!fp_is_equal(vardata[varID][offset + i], missval) && array[i] >= fltarr[index]
                             && array[i] < fltarr[index + 1])
                           {
                             vardata[varID][offset + i] += array[i];
