@@ -17,7 +17,7 @@
 #include "griddes.h"
 #include "grid_point_search.h"
 #include <mpim_grid.h>
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "verifygrid.h"
 #include "field_functions.h"
 
@@ -177,7 +177,7 @@ is_zonal_grid(int gridID, const std::vector<double> &xbounds2d, const std::vecto
       isZonalGrid = true;
       for (size_t i = 1; i < nlat; ++i)
         {
-          if (IS_NOT_EQUAL(ybounds2d[2 * i - 1], ybounds2d[2 * i]))
+          if (is_not_equal(ybounds2d[2 * i - 1], ybounds2d[2 * i]))
             {
               isZonalGrid = false;
               break;
@@ -590,13 +590,13 @@ remap_kernel(int operfunc, const Varray<size_t> &indices, size_t &numMissVals2,
         {
           auto v1 = vec1[indices[k]];
           fieldvec[k] = v1;
-          if (dbl_is_equal(v1, missval)) field.numMissVals++;
+          if (fp_is_equal(v1, missval)) field.numMissVals++;
         }
 
       field.size = nvalues;
       field.missval = missval;
       value = field_function(field, operfunc);
-      if (dbl_is_equal(value, missval)) numMissVals2++;
+      if (fp_is_equal(value, missval)) numMissVals2++;
     }
   else
     {
