@@ -349,7 +349,7 @@ public:
         if (Options::cdoVerbose)
           {
             auto areasize = gridcellArea.size();
-            auto mmm = varray_min_max_mean(areasize, gridcellArea);
+            auto mmm = varray_min_max_mean(gridcellArea, areasize);
             cdo_print("gridcellAreas: %zu %#12.5g%#12.5g%#12.5g", areasize, mmm.min, mmm.mean, mmm.max);
           }
       }
@@ -374,7 +374,7 @@ public:
         streamClose(streamID);
 
         for (size_t i = 0; i < masksize; ++i)
-          if (dbl_is_equal(gridmask[i], missval)) gridmask[i] = 0;
+          if (fp_is_equal(gridmask[i], missval)) gridmask[i] = 0;
       }
     else if (operatorID == USEGRIDNUMBER)
       {
