@@ -19,8 +19,19 @@
 #include "util_files.h"
 #include "util_string.h"
 
-class ModuleSplitrec
+class Splitrec : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Splitrec",
+    .operators = { { "splitrec", SplitHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, OBASE, OnlyFirst },
+  };
+  inline static RegisterEntry<Splitrec> registration = RegisterEntry<Splitrec>(module);
 
   CdoStreamID streamID1;
   int vlistID1;
@@ -31,12 +42,8 @@ class ModuleSplitrec
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    if (process_self().m_ID != 0) cdo_abort("This operator can't be combined with other operators!");
-
     operator_check_argc(0);
 
     dataIsUnchanged = data_is_unchanged();
@@ -46,10 +53,11 @@ public:
 
     fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
     int index = 0;
     int tsID = 0;
@@ -87,7 +95,7 @@ public:
             if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
             else
               {
-                field.init(varList1[varID]);
+                field.init(varList1.vars[varID]);
                 cdo_read_record(streamID1, field);
                 cdo_write_record(streamID2, field);
               }
@@ -101,19 +109,8 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
-    cdo_finish();
   }
 };
-
-void *
-Splitrec(void *process)
-{
-  ModuleSplitrec splitrec;
-  splitrec.init(process);
-  splitrec.run();
-  splitrec.close();
-  return nullptr;
-}
