@@ -21,7 +21,7 @@
 static void
 var_rms(const Varray<double> &w, const FieldVector &field1, const FieldVector &field2, Field &field3)
 {
-  auto is_EQ = dbl_is_equal;
+  auto is_EQ = fp_is_equal;
   auto grid1 = field1[0].grid;
   auto grid2 = field2[0].grid;
   auto missval1 = field1[0].missval;
