@@ -23,8 +23,6 @@
 
 #include <cdi.h>
 
-#include <utility>
-
 #include "cdo_options.h"
 #include "cdo_vlist.h"
 #include "process_int.h"
@@ -69,8 +67,32 @@ define_reduced_grid(int gridID1, size_t ysize, std::vector<int> &hpReducedPoints
   return gridID;
 }
 
-class ZonStat
+class Zonstat : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Zonstat",
+    .operators = { { "zonrange", FieldFunc_Range, 0, ZonstatHelp },
+                   { "zonmin", FieldFunc_Min, 0, ZonstatHelp },
+                   { "zonmax", FieldFunc_Max, 0, ZonstatHelp },
+                   { "zonsum", FieldFunc_Sum, 0, ZonstatHelp },
+                   { "zonmean", FieldFunc_Mean, 0, ZonstatHelp },
+                   { "zonavg", FieldFunc_Avg, 0, ZonstatHelp },
+                   { "zonstd", FieldFunc_Std, 0, ZonstatHelp },
+                   { "zonstd1", FieldFunc_Std1, 0, ZonstatHelp },
+                   { "zonvar", FieldFunc_Var, 0, ZonstatHelp },
+                   { "zonvar1", FieldFunc_Var1, 0, ZonstatHelp },
+                   { "zonskew", FieldFunc_Skew, 0, ZonstatHelp },
+                   { "zonkurt", FieldFunc_Kurt, 0, ZonstatHelp },
+                   { "zonmedian", FieldFunc_Median, 0, ZonstatHelp },
+                   { "zonpctl", FieldFunc_Pctl, 0, ZonstatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Zonstat> registration = RegisterEntry<Zonstat>(module);
   int gridIDdestroy = -1, gridID1 = -1, gridID2 = -1;
   int zongridID = -1;
   int sourceGridIsRegular = true;
@@ -94,38 +116,15 @@ class ZonStat
 
   double pn = 0.0;
 
-  void
-  add_operators(void)
-  {
-    // clang-format off
-    cdo_operator_add("zonmin",    FieldFunc_Min,    0, nullptr);
-    cdo_operator_add("zonmax",    FieldFunc_Max,    0, nullptr);
-    cdo_operator_add("zonrange",  FieldFunc_Range,  0, nullptr);
-    cdo_operator_add("zonsum",    FieldFunc_Sum,    0, nullptr);
-    cdo_operator_add("zonmean",   FieldFunc_Mean,   0, nullptr);
-    cdo_operator_add("zonavg",    FieldFunc_Avg,    0, nullptr);
-    cdo_operator_add("zonvar",    FieldFunc_Var,    0, nullptr);
-    cdo_operator_add("zonvar1",   FieldFunc_Var1,   0, nullptr);
-    cdo_operator_add("zonstd",    FieldFunc_Std,    0, nullptr);
-    cdo_operator_add("zonstd1",   FieldFunc_Std1,   0, nullptr);
-    cdo_operator_add("zonskew",   FieldFunc_Skew,   0, nullptr);
-    cdo_operator_add("zonkurt",   FieldFunc_Kurt,   0, nullptr);
-    cdo_operator_add("zonmedian", FieldFunc_Median, 0, nullptr);
-    cdo_operator_add("zonpctl",   FieldFunc_Pctl,   0, nullptr);
-    // clang-format on
-  }
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    add_operators();
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
 
+    auto lminmax = (operfunc == FieldFunc_Min || operfunc == FieldFunc_Max);
+
     if (operfunc == FieldFunc_Pctl)
       {
         operator_input_arg("percentile number");
@@ -140,21 +139,22 @@ public:
         if (!gridInqYbounds(gridID2, NULL)) cdo_abort("Target grid cell bounds missing!");
         if (gridInqXsize(gridID2) > 1) cdo_abort("Target grid must be zonal!");
       }
-    else
-      {
-        operator_check_argc(0);
-      }
+    else { operator_check_argc(0); }
 
     streamID1 = cdo_open_read(0);
 
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = vlistDuplicate(vlistID1);
 
+    varList1 = VarList(vlistID1);
+
+    if (!lminmax) vlist_unpack(vlistID2);
+
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     for (int index = 0; index < ngrids; ++index)
       {
         auto gridID = vlistGrid(vlistID1, index);
@@ -220,18 +220,15 @@ public:
 
     if (Options::cdoChunkType == CDI_UNDEFID)
       {
-        auto nvars = vlistNvars(vlistID2);
-        for (int varID = 0; varID < nvars; ++varID) cdiDefKeyInt(vlistID2, varID, CDI_KEY_CHUNKTYPE, CDI_CHUNK_AUTO);
+        for (const auto &var : varList1.vars) cdiDefKeyInt(vlistID2, var.ID, CDI_KEY_CHUNKTYPE, CDI_CHUNK_AUTO);
       }
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varListInit(varList1, vlistID1);
-
     nlatmax = gridInqYsize(gridID2);
 
-    auto zonVar = varList1[0];
+    auto zonVar = varList1.vars[0];
     zonVar.gridID = gridID2;
     zonVar.gridsize = nlatmax;
     zonVar.nlevels = 1;
@@ -256,9 +253,9 @@ public:
 
     for (int recID = 0; recID < nrecs; ++recID)
       {
-        int varID, levelID;
-        cdo_inq_record(streamID1, &varID, &levelID);
-        field1.init(varList1[varID]);
+        auto [varID, levelID] = cdo_inq_record(streamID1);
+        const auto &var1 = varList1.vars[varID];
+        field1.init(var1);
         cdo_read_record(streamID1, field1);
         field1.grid = gridID1;
 
@@ -267,13 +264,10 @@ public:
         if (zongridID != -1 && zongridID == field1.grid) { field_ncopy(nlatmax, field1, field2); }
         else if (sourceGridIsRegular)
           {
-            if (is_healpix_grid(varList1[varID].gridID)) reorder_field(field1, hpRingIndices);
+            if (is_healpix_grid(var1.gridID)) reorder_field(field1, hpRingIndices);
             (operfunc == FieldFunc_Pctl) ? zonal_pctl(field1, field2, pn) : zonal_function(field1, field2, operfunc);
           }
-        else
-          {
-            remap_zonal_mean(remapIndices, remapWeights, field1, field2);
-          }
+        else { remap_zonal_mean(remapIndices, remapWeights, field1, field2); }
 
         cdo_def_record(streamID2, varID, levelID);
         cdo_write_record(streamID2, field2);
@@ -281,7 +275,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -293,25 +287,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
-
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
     if (gridIDdestroy != -1) gridDestroy(gridIDdestroy);
-
-    cdo_finish();
   }
 };
-
-void *
-Zonstat(void *process)
-{
-  ZonStat zonstat;
-  zonstat.init(process);
-  zonstat.run();
-  zonstat.close();
-
-  return nullptr;
-}
