@@ -1,9 +1,15 @@
 #include <cdi.h>
-#include <cstring>
+
 #include <cstdlib>
 #include <vector>
 
-#include "cdi_int.h"
+//#include "cdi_int.h"
+extern "C" {
+void cdiDefTableID(int tableID);
+void gridGenXvals(int xsize, double xfirst, double xlast, double xinc, double *xvals);
+void gridGenYvals(int gridtype, int ysize, double yfirst, double ylast, double yinc, double *yvals);
+}
+
 #include "cdo_default_values.h"
 #include "cdo_cdi_wrapper.h"
 #include "cdo_output.h"
@@ -101,8 +107,6 @@ get_steptype_name(int tsteptype)
   return "unknown";
 }
 
-
-
 }  // namespace cdo
 
 int
@@ -215,4 +219,4 @@ get_healpix_params(int gridID)
   return std::make_pair(nside, hpOrder);
 }
 
-}
+}  // namespace cdo
