@@ -38,9 +38,9 @@ extern int nsb;  // Number of significant bits
 extern bool benchmark;
 extern bool silentMode;
 extern bool test;
+extern bool force;
 
-extern int filterId;
-extern std::vector<int> filterParams;
+extern std::string filterSpec;
 
 extern int cdoShuffle;
 extern bool cdoCompress;
