@@ -13,6 +13,7 @@
 
 #include <cdi.h>
 
+#include "cdo_timer.h"
 #include "process_int.h"
 
 #define TO_KELVIN(x) ((x) + 273.15)
@@ -114,7 +115,7 @@ write_file(const char path[], const double array[], int length)
       streamDefTimestep(streamID, tsID);
 
       auto value = array[tsID];
-      size_t numMissVals = dbl_is_equal(value, MISSVAL) ? 1 : 0;
+      size_t numMissVals = fp_is_equal(value, MISSVAL) ? 1 : 0;
 
       streamWriteVar(streamID, varID, &value, numMissVals);
     }
@@ -172,25 +173,25 @@ testEcaFd()
 
   submit_cdo_command("eca_fd in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 
   write_file("in.srv", array, 3);
 
   submit_cdo_command("eca_fd in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 0.0));
+  assert(fp_is_equal(vars[0][0], 0.0));
 
   write_file("in.srv", array, 5);
 
   submit_cdo_command("eca_fd in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.));
+  assert(fp_is_equal(vars[0][0], 1.));
 
   write_file("in.srv", array, 6);
 
   submit_cdo_command("eca_fd in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 2.));
+  assert(fp_is_equal(vars[0][0], 2.));
 }
 
 static void
@@ -207,21 +208,21 @@ testEcaSu()
 
   submit_cdo_command("eca_su in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 
   write_file("in.srv", array, 6);
 
   submit_cdo_command("eca_su in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 2.));
+  assert(fp_is_equal(vars[0][0], 2.));
 
   submit_cdo_command("eca_su,20.0 in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 4.));
+  assert(fp_is_equal(vars[0][0], 4.));
 
   submit_cdo_command("eca_su,30.0 in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 0.));
+  assert(fp_is_equal(vars[0][0], 0.));
 }
 
 static void
@@ -240,42 +241,42 @@ testFdns()
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 
   write_file("in1.srv", array1, 2);
   write_file("in2.srv", array2, 2);
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 0.));
+  assert(fp_is_equal(vars[0][0], 0.));
 
   write_file("in1.srv", array1, 3);
   write_file("in2.srv", array2, 3);
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 0.));
+  assert(fp_is_equal(vars[0][0], 0.));
 
   write_file("in1.srv", array1, 4);
   write_file("in2.srv", array2, 4);
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.));
+  assert(fp_is_equal(vars[0][0], 1.));
 
   write_file("in1.srv", array1, 5);
   write_file("in2.srv", array2, 5);
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.));
+  assert(fp_is_equal(vars[0][0], 1.));
 
   write_file("in1.srv", array1 + 4, 1);
   write_file("in2.srv", array2 + 4, 1);
 
   submit_cdo_command("fdns in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 }
 
 static void
@@ -296,34 +297,34 @@ testEcaGsl()
 
   submit_cdo_command("eca_gsl in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 7.0));
-  assert(dbl_is_equal(vars[1][0], 181.0));
+  assert(fp_is_equal(vars[0][0], 7.0));
+  assert(fp_is_equal(vars[1][0], 181.0));
 
   //  submit_cdo_command("eca_gsl,6,5.0,0.6 in1.srv in2.srv out.srv");
   //  read_file("out.srv", vars, nvars, nts);
-  //  assert(dbl_is_equal(vars[0][0], MISSVAL));
-  //  assert(dbl_is_equal(vars[1][0], MISSVAL));
+  //  assert(fp_is_equal(vars[0][0], MISSVAL));
+  //  assert(fp_is_equal(vars[1][0], MISSVAL));
 
   write_file("in1.srv", array1, 7);
 
   submit_cdo_command("eca_gsl in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.0));
-  assert(dbl_is_equal(vars[1][0], 181.0));
+  assert(fp_is_equal(vars[0][0], 1.0));
+  assert(fp_is_equal(vars[1][0], 181.0));
 
   write_file("in1.srv", array1, 8);
 
   submit_cdo_command("eca_gsl in1.srv in2.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 2.0));
-  assert(dbl_is_equal(vars[1][0], 181.0));
+  assert(fp_is_equal(vars[0][0], 2.0));
+  assert(fp_is_equal(vars[1][0], 181.0));
 
   //  write_file("in1.srv", array1, 4);
 
   //  submit_cdo_command("eca_gsl in1.srv in2.srv out.srv");
   //  read_file("out.srv", vars, nvars, nts);
-  //  assert(dbl_is_equal(vars[0][0], MISSVAL));
-  //  assert(dbl_is_equal(vars[1][0], MISSVAL));
+  //  assert(fp_is_equal(vars[0][0], MISSVAL));
+  //  assert(fp_is_equal(vars[1][0], MISSVAL));
 }
 /*
 static
@@ -367,25 +368,25 @@ testTimcount()
 
   submit_cdo_command("timcount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 
   write_file("in.srv", array, 3);
 
   submit_cdo_command("timcount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.));
+  assert(fp_is_equal(vars[0][0], 1.));
 
   write_file("in.srv", array, 5);
 
   submit_cdo_command("timcount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 2.));
+  assert(fp_is_equal(vars[0][0], 2.));
 
   write_file("in.srv", array, 6);
 
   submit_cdo_command("timcount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 3.));
+  assert(fp_is_equal(vars[0][0], 3.));
 }
 /*
 static
@@ -404,25 +405,25 @@ void testSeascount()
 
   submit_cdo_command("seascount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], MISSVAL));
+  assert(fp_is_equal(vars[0][0], MISSVAL));
 
   write_file("in.srv", array, 3);
 
   submit_cdo_command("seascount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 1.));
+  assert(fp_is_equal(vars[0][0], 1.));
 
   write_file("in.srv", array, 5);
 
   submit_cdo_command("seascount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 2.));
+  assert(fp_is_equal(vars[0][0], 2.));
 
   write_file("in.srv", array, 6);
 
   submit_cdo_command("seascount in.srv out.srv");
   read_file("out.srv", vars, nvars, nts);
-  assert(dbl_is_equal(vars[0][0], 3.));
+  assert(fp_is_equal(vars[0][0], 3.));
 }
 */
 static void
