@@ -53,12 +53,13 @@ check_xyvals(size_t nvals, double *xvals, double *yvals)
 static std::string
 gen_param(const char *fmt, ...)
 {
-  va_list args;
-  char str[256];
+  constexpr size_t len = 256;
+  char str[len];
 
+  va_list args;
   va_start(args, fmt);
 
-  vsprintf(str, fmt, args);
+  vsnprintf(str, len, fmt, args);
 
   va_end(args);
 
@@ -149,7 +150,7 @@ do_proj_inv(const char *params, size_t nvals, double *xvals, double *yvals)
 static inline bool
 checkValIsMiss(const char *projection, const char *name, double val, double missval)
 {
-  if (IS_EQUAL(val, missval))
+  if (is_equal(val, missval))
     {
       cdo_warning("%s mapping parameter %s missing!", projection, name);
       return true;
@@ -288,7 +289,7 @@ cdo_lcc_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
                         || checkValIsMiss(projection, "latitude_of_projection_origin", gpp.lat_0, gpp.mv)
                         || checkValIsMiss(projection, "standard_parallel", gpp.lat_1, gpp.mv);
 
-  if (!paramIsMissing && IS_EQUAL(gpp.x_0, gpp.mv) && IS_EQUAL(gpp.y_0, gpp.mv) && is_not_equal(gpp.xval_0, gpp.mv)
+  if (!paramIsMissing && is_equal(gpp.x_0, gpp.mv) && is_equal(gpp.y_0, gpp.mv) && is_not_equal(gpp.xval_0, gpp.mv)
       && is_not_equal(gpp.yval_0, gpp.mv))
     {
 #ifdef HAVE_LIBPROJ
@@ -409,7 +410,7 @@ cdo_stere_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
                         || checkValIsMiss(projection, "latitude_of_projection_origin", gpp.lat_0, gpp.mv)
                         || checkValIsMiss(projection, "standard_parallel", gpp.lat_1, gpp.mv);
 
-  if (!paramIsMissing && IS_EQUAL(gpp.x_0, gpp.mv) && IS_EQUAL(gpp.y_0, gpp.mv) && is_not_equal(gpp.xval_0, gpp.mv)
+  if (!paramIsMissing && is_equal(gpp.x_0, gpp.mv) && is_equal(gpp.y_0, gpp.mv) && is_not_equal(gpp.xval_0, gpp.mv)
       && is_not_equal(gpp.yval_0, gpp.mv))
     {
 #ifdef HAVE_LIBPROJ
@@ -610,22 +611,6 @@ cdo_proj_to_lonlat(char *proj_params, size_t nvals, double *xvals, double *yvals
 double
 gridGetPlanetRadius(int gridID)
 {
-  double planetRadius = 0.0;
-
-  auto gridtype = gridInqType(gridID);
-  auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-  if (projtype == CDI_PROJ_LCC)
-    {
-      CDI_GridProjParams gpp;
-      gridInqParamsLCC(gridID, &gpp);
-      if (IS_EQUAL(gpp.b, gpp.mv)) planetRadius = gpp.a;
-    }
-  else if (projtype == CDI_PROJ_STERE)
-    {
-      CDI_GridProjParams gpp;
-      gridInqParamsSTERE(gridID, &gpp);
-      if (IS_EQUAL(gpp.b, gpp.mv)) planetRadius = gpp.a;
-    }
-
-  return planetRadius;
+  double radius{ 0 };
+  return (CDI_NOERR == cdiInqAttFlt(gridID, CDI_GLOBAL, "earth_radius", 1, &radius) && radius > 1) ? radius : 0.0;
 }
